import logging
import time
import uuid
import os

from .emproducer import push_to_queue
from .eswrapper import index_to_es

log = logging.getLogger('file')
anu_etl_wf_error_topic = os.environ.get('ANU_ETL_WF_ERROR_TOPIC', 'anuvaad-etl-wf-errors')


def post_error(code, message, cause):
    error = {
        "errorID": generate_error_id(),
        "code": code,
        "message": message,
        "timeStamp": eval(str(time.time()).replace('.', '')),
        "errorType": "core-error"
    }
    if cause is not None:
        error["cause"] = cause

    index_to_es(error)
    log.info("Error posted to the es index.")

    return error


def post_error_wf(code, message, jobId, taskId, state, status, cause):
    error = {
        "errorID": generate_error_id(),
        "code": code,
        "message": message,
        "timeStamp": eval(str(time.time()).replace('.', '')),
        "jobID": jobId,
        "taskID": taskId,
        "state": state,
        "status": status,
        "errorType": "wf-error"
    }
    if cause is not None:
        error["cause"] = cause
    push_to_queue(error, anu_etl_wf_error_topic)
    log.info("Error pushed to the wf error topic.")
    index_to_es(error)
    log.info("Error posted to the es index.")
    return error


def generate_error_id():
    return str(uuid.uuid4())



import logging
import os
from datetime import datetime

from elasticsearch import Elasticsearch

log = logging.getLogger('file')

es_url = os.environ.get('ES_URL', 'http://172.30.0.55:9200')
es_error_index_test = "anuvaad-etl-errors-test-v1"
es_core_error_index = "anuvaad-etl-errors-core-v1"
es_wf_error_index = "anuvaad-etl-errors-wf-v1"
es_error_type = "errors"


def instantiate_es_client():
    es_client = Elasticsearch([es_url])
    return es_client


def index_to_es(index_obj):
    try:
        es = instantiate_es_client()
        id = index_obj["errorID"]
        if index_obj["errorType"] == "core-error":
            in_name = es_core_error_index
        else:
            in_name = es_wf_error_index
        index_obj = add_timestamp_field(index_obj)
        es.index(index=in_name, doc_type=es_error_type, id=id, body=index_obj)
    except Exception as e:
        log.exception("Indexing FAILED for errorID: " + index_obj["errorID"])


def add_timestamp_field(error):
    date_format = "%Y-%m-%d'T'%H:%M:%S.%f'Z'"
    epoch = error["timeStamp"]
    epoch_short = eval((str(epoch)[:10]))
    final_date = datetime.fromtimestamp(epoch_short).strftime(date_format)
    error["@timestamp"] = final_date
    return error


from distutils.core import setup

long_description = "A package to standardize and manage errors in anuvaad dataflow pipeline by EkStep \n" \
                   "Visit the following repo for more details: https://github.com/project-anuvaad/anuvaad-em"

setup(
  name = 'anuvaad_em',         # How you named your package folder
  packages = ['anuvaad_em'],   # Chose the same as "name"
  version = '0.0.1',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'A package to standardize and manage errors in anuvaad dataflow pipeline by Tarento',   # Give a short description about your library
  author = 'Vishal Mahuli',                   # Type in your name
  author_email = 'vishal.mahuli@tarento.com',      # Type in your E-Mail
  url = 'https://www.tarento.com',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/project-anuvaad/anuvaad-em/archive/0.0.1.tar.gz',    # Release source.tar.gz asset
  keywords = ['ANUVAAD', 'ERROR', 'MANAGER', 'TARENTO', 'EKSTEP', 'SUVAAS'],   # Keywords that define your package best
  long_description=long_description,
  install_requires=[            # Packages to be explicitly installed ONLY. (No inbuilt py packages)
          'kafka',
          'kafka-python',
          'uuid',
          'datetime',
          'elasticsearch',
  ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3.7',      #Specify which pyhton versions that you want to support
  ],
)


import json
import logging

import os
from kafka import KafkaProducer


log = logging.getLogger('file')
cluster_details = os.environ.get('KAFKA_CLUSTER_DETAILS', 'localhost:9092')


def instantiate():
    producer = KafkaProducer(bootstrap_servers=[cluster_details],
                             api_version=(1, 0, 0),
                             value_serializer=lambda x: json.dumps(x).encode('utf-8'))
    return producer


# Method to push records to a topic in the kafka queue
def push_to_queue(object_in, topic):
    producer = instantiate()
    try:
        producer.send(topic, value=object_in)
        log.info("Pushing to the topic: " + topic)
        producer.flush()
    except Exception as e:
        log.exception("Exception while producing: " + str(e))
